#!/usr/bin/python
# -*- coding: utf-8 -*-

PROGRAM = 'bittivahti'
VERSION = '0.9.3'
DEVFILE = '/proc/net/dev'

import re
import sys
import time

from .utils import clear, pretty_unit

# Python 3 int is long
if sys.version > '3':
    long = int


class Bittivahti:
    device = {}
    delta = {}
    total = {}
    start = time.time()
    period = None

    def update_state(self):
        lines = None
        # Read network traffic stats to memory quickly
        with open(DEVFILE, 'r') as f:
            lines = f.readlines()[2:]
            curtime = time.time()
            self.period = curtime - self.start
            self.start = curtime

        for line in lines:
            data = re.split('[ \t:]+', line.strip())
            iface = data[0]
            rx, rxp = map(long, data[1:3])
            tx, txp = map(long, data[9:11])
            trafficdata = [rx, tx, rxp, txp]

            if rx > 0 or tx > 0:
                if iface in self.device:
                    self.delta[iface] = [b-a for a, b in zip(self.device[iface], trafficdata)]
                else:
                    self.delta[iface] = [long(0)]*4
                    self.total[iface] = [long(0)]*4
                self.device[iface] = trafficdata

                # Calculate self.total amount of traffic
                if True in [a < 0 for a in self.delta[iface]]:
                    pass  # ignore updates where bytes or packets is negative
                else:
                    self.total[iface] = [a+b for a, b in zip(self.total[iface], self.delta[iface])]

    def printdata(self):
        '''Print current state'''
        print("{program} {version}".format(program=PROGRAM, version=VERSION))
        print("interface   |      RX bw / packets |      TX bw / packets | "
              "total:  RX       TX ")
        print("\n".join(self.format_data()))

    def format_data(self):
        '''Format traffic statistics to a display array of lines'''
        display = []
        for iface in sorted(self.device.keys()):
            rx, tx, rxp, txp = [x/self.period for x in self.delta[iface]]
            rx_t, tx_t, rxp_t, txp_t = self.total[iface]
            d = {'iface': iface,
                 'rx': pretty_unit(rx),
                 'tx': pretty_unit(tx),
                 'rxp': pretty_unit(rxp, minunit=1, format="%0.0f"),
                 'txp': pretty_unit(txp, minunit=1, format="%0.0f"),
                 'rx_t': pretty_unit(rx_t),
                 'tx_t': pretty_unit(tx_t)
                 }
            display.append((
                   "%(iface)-12s| %(rx)7sB/s %(rxp)6sp/s |"
                   " %(tx)7sB/s %(txp)6sp/s |"
                   "   %(rx_t)7sB %(tx_t)7sB") % d)
        return display

    def loop(self, sleep, dynunit, colours):
        print('Please wait. The display is updated every {sleep:.0f} seconds.'
              .format(sleep=sleep))
        print('Starting up...')
        self.update_state()
        time.sleep(sleep)
        # Main loop
        while True:
            try:
                clear()
                self.update_state()
                self.printdata()
                time.sleep(sleep)
            except KeyboardInterrupt:
                raise SystemExit('\n\nBye!')



import click

from .bittivahti import Bittivahti


@click.command('bittivahti')
@click.option('--curses/--no-curses', help='EXPERIMENTAL curses mode')
@click.option('-i', '--interval', default=3, type=click.FLOAT,
              help='Wait SECONDS between updates', metavar='SECONDS')
def main(interval, curses):
    """Display traffic statistics on local network interfaces"""
    dynunit = False
    colors = False
    if curses:
        from .terminal import main
        main(update_delay=interval)
    else:
        bitti = Bittivahti()
        bitti.loop(interval, dynunit, colors)



import curses
import time
import signal
from datetime import datetime

from .bittivahti import Bittivahti

bitti = Bittivahti()


class TimedOutException(Exception):
    pass


def _timeout_handler(signum, frame):
    raise TimedOutException()

signal.signal(signal.SIGALRM, _timeout_handler)
IN_TIMEOUT = -1


def main(update_delay=3):
    try:
        curses.wrapper(loop, update_delay)
    except KeyboardInterrupt:
        pass



def loop(screen, update_delay):
    redraw(screen)
    #print('Please wait. The display is updated every {sleep:.0f} seconds.'
    #         .format(sleep=sleep))
    #print('Starting up...')

    # Main loop
    y, x = screen.getmaxyx()
    while True:
        try:
            signal.alarm(int(update_delay))
            in_char = screen.getch()  # Input
            signal.alarm(0)
        except TimedOutException:
            in_char = IN_TIMEOUT

        # Handle resize
        resized = curses.is_term_resized(y, x)
        if resized is True:
            y, x = screen.getmaxyx()
            screen.clear()
            curses.resizeterm(y, x)
            redraw(screen)

        if in_char in [ord(' '), IN_TIMEOUT]:
            redraw(screen)
        elif in_char is ord('q'):
            raise SystemExit()
        else:
            pass  # ignore

def redraw(screen):
    bitti.update_state()
    display_lines = bitti.format_data()
    screen.clear()
    #screen.immedok(True)
    screen.border(0)
    screen.refresh()
    title = "bittivahti 0.93 (press q to quit)"
    title_label = curses.newwin(1, len(title)+1, 0, 2)
    title_label.immedok(True)
    title_label.addstr(title)

    content_box = curses.newwin(20, 80, 2, 2)
    content_box.immedok(True)
    content_box.addstr("\n".join(display_lines))

    cur_time = "{now}".format(now=datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
    time_box = curses.newwin(1, len(cur_time)+1, 0, 80-len(cur_time)+1)
    time_box.immedok(True)
    time_box.addstr(cur_time)



#!/usr/bin/env python
from setuptools import setup, find_packages
import os.path

# Read README for long description
BASE = os.path.dirname(__file__)
README = os.path.join(BASE, 'README.rst')
with open(README) as f:
    long_description = f.read()


setup(name='bittivahti',
      version='0.9.3',
      description="Bittivahti bandwidth monitor for Linux",
      long_description=long_description,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Internet',
          'Topic :: System :: Networking',
          'Topic :: Utilities',
      ],
      keywords='',
      author='Joonas Kuorilehto',
      author_email='joneskoo@kapsi.fi',
      url='https://github.com/joneskoo/bittivahti',
      license='MIT',
      packages=find_packages(exclude=['tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'click'
      ],
      extras_require={
          'test': ['pytest'],
      },
      entry_points="""
      [console_scripts]
      bittivahti=bittivahti.cli:main
      """
      )



import sys


def clear():
    sys.stdout.write("\x1b[H\x1b[2J")


def pretty_unit(value, base=1000, minunit=None, format="%0.1f"):
    '''Finds the correct unit and returns a pretty string

    pretty_unit(4190591051, base=1024) = "3.9 Gi"
    '''
    if not minunit:
        minunit = base

    # Units based on base
    if base == 1000:
        units = [' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y']
    elif base == 1024:
        units = ['  ', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi']
    else:
        raise ValueError("The unit base has to be 1000 or 1024")

    # Divide until below threshold or base
    v = float(value)
    u = base
    for unit in units:
        if v >= base or u <= minunit:
            v = v/base
            u = u * base
        else:
            if v >= 10:
                return "%0.0f %s" % (v, unit)
            else:
                return format % v + units[0] + unit
